@@ -9,8 +9,7 @@
 #ifndef GT_DNA_STRING_H_
 #define GT_DNA_STRING_H_
 
-#include "gt_commons.h"
-#include "gt_map.h"
+#include "gt_essentials.h"
 
 #define GT_DNA_RANGE 5
 
@@ -22,6 +21,9 @@
 
 #define gt_dna_string gt_string
 
+// Orientation (strand)
+typedef enum { FORWARD, REVERSE, UNKNOWN } gt_strand;
+
 typedef struct {
   gt_dna_string* dna_string;
   uint64_t current_pos;
@@ -36,12 +38,14 @@ extern const bool gt_dna[256];
 extern const bool gt_iupac_code[256];
 
 extern const char gt_dna_normalized[256];
+extern const char gt_dna_strictly_normalized[256];
 extern const char gt_complement_table[256];
 
 #define gt_is_dna(character)         (gt_dna[(int)(character)])
 #define gt_is_iupac_code(character)  (gt_iupac_code[(int)(character)])
 
 #define gt_get_dna_normalized(character) (gt_dna_normalized[(int)(character)])
+#define gt_get_dna_strictly_normalized(character) (gt_dna_strictly_normalized[(int)(character)])
 #define gt_get_complement(character) (gt_complement_table[(int)(character)])
 
 /*
@@ -78,6 +82,7 @@ GT_INLINE void gt_dna_string_set_nstring(gt_dna_string* const dna_string,char* c
 
 GT_INLINE void gt_dna_string_reverse_complement(gt_dna_string* const dna_string);
 GT_INLINE void gt_dna_string_reverse_complement_copy(gt_dna_string* const dna_string_dst,gt_dna_string* const dna_string_src);
+GT_INLINE gt_dna_string* gt_dna_string_reverse_complement_dup(gt_dna_string* const dna_string);
 
 /*
  * DNA String Iterator
