@@ -28,6 +28,10 @@ typedef struct {
 typedef struct {
   gt_ihash_element* ihash_head;
 } gt_ihash;
+typedef struct {
+  gt_ihash_element* ihash;
+  gt_ihash_element* next;
+} gt_ihash_iterator;
 
 /*
  * Constructor
@@ -35,6 +39,7 @@ typedef struct {
 GT_INLINE gt_ihash* gt_ihash_new(void);
 GT_INLINE void gt_ihash_clear(gt_ihash* const ihash,const bool free_element);
 GT_INLINE void gt_ihash_delete(gt_ihash* const ihash,const bool free_element);
+GT_INLINE void gt_ihash_destroy(gt_ihash* const ihash);
 
 /*
  * Basic (Type-unsafe) Accessors
@@ -62,6 +67,7 @@ GT_INLINE uint64_t gt_ihash_get_num_elements(gt_ihash* const ihash);
  */
 GT_INLINE gt_ihash* gt_ihash_dup(gt_ihash* const ihash);
 GT_INLINE void gt_ihash_copy(gt_ihash* const ihash_dst,gt_ihash* const ihash_src);
+GT_INLINE void gt_ihash_sort_by_key(gt_ihash* const ihash);
 
 /*
  * Iterator
@@ -69,8 +75,15 @@ GT_INLINE void gt_ihash_copy(gt_ihash* const ihash_dst,gt_ihash* const ihash_src
 #define GT_IHASH_BEGIN_ITERATE(ihash,it_ikey,it_element,type) { \
   gt_ihash_element *ihash_##ih_element, *ihash_##tmp; \
   HASH_ITER(hh,ihash->ihash_head,ihash_##ih_element,ihash_##tmp) { \
-    register type* const it_element = (type*)(ihash_##ih_element->element); \
-    register int64_t const it_ikey = ihash_##ih_element->key;
+    type* const it_element = (type*)(ihash_##ih_element->element); \
+    int64_t const it_ikey = ihash_##ih_element->key;
 #define GT_IHASH_END_ITERATE }}
 
+GT_INLINE gt_ihash_iterator* gt_ihash_iterator_new(gt_ihash* const ihash);
+GT_INLINE void gt_ihash_iterator_delete(gt_ihash_iterator* const ihash_iterator);
+
+GT_INLINE bool gt_ihash_iterator_next(gt_ihash_iterator* const ihash_iterator);
+GT_INLINE int64_t gt_ihash_iterator_get_key(gt_ihash_iterator* const ihash_iterator);
+GT_INLINE void* gt_ihash_iterator_get_element(gt_ihash_iterator* const ihash_iterator);
+
 #endif /* GT_IHASH_H_ */
