@@ -7,6 +7,10 @@
  */
 
 #include "gt_hash.h"
+#include "gt_commons.h"
+#include "gt_error.h"
+#include "gt_mm.h"
+#include "gt_string.h"
 
 /*
  * Setup
@@ -15,7 +19,7 @@ GT_INLINE void gt_shash_free_element(gt_shash_element* const shash_element) {
   GT_NULL_CHECK(shash_element);
   switch (shash_element->element_type) {
     case GT_HASH_TYPE_REGULAR:
-      free(shash_element->element);
+      gt_free(shash_element->element);
       break;
     case GT_HASH_TYPE_OBJECT:
       shash_element->element_setup.element_free_fx(shash_element->element);
@@ -28,19 +32,18 @@ GT_INLINE void gt_shash_free_element(gt_shash_element* const shash_element) {
 GT_INLINE void gt_shash_free_shash_element(gt_shash_element* const shash_element,const bool free_element) {
   GT_NULL_CHECK(shash_element);
   // Free key
-  free(shash_element->key);
+  gt_free(shash_element->key);
   // Free element
   if (free_element) gt_shash_free_element(shash_element);
   // Free handler
-  free(shash_element);
+  gt_free(shash_element);
 }
 
 /*
  * Constructor
  */
 GT_INLINE gt_shash* gt_shash_new(void) {
-  gt_shash* shash=malloc(sizeof(gt_shash));
-  gt_cond_fatal_error(!shash,MEM_HANDLER);
+  gt_shash* shash = gt_alloc(gt_shash);
   shash->shash_head = NULL; // uthash initializer
   return shash;
 }
@@ -55,7 +58,12 @@ GT_INLINE void gt_shash_clear(gt_shash* const shash,const bool free_element) {
 GT_INLINE void gt_shash_delete(gt_shash* const shash,const bool free_element) {
   GT_HASH_CHECK(shash);
   gt_shash_clear(shash,free_element);
-  free(shash);
+  gt_free(shash);
+}
+GT_INLINE void gt_shash_destroy(gt_shash* const shash) {
+  GT_HASH_CHECK(shash);
+  gt_shash_clear(shash,true);
+  gt_free(shash);
 }
 
 /*
@@ -73,11 +81,10 @@ GT_INLINE char* gt_shash_insert_primitive(
   GT_HASH_CHECK(shash);
   GT_ZERO_CHECK(element_size);
   GT_NULL_CHECK(key); GT_NULL_CHECK(element);
-  register gt_shash_element *shash_element = gt_shash_get_shash_element(shash,key);
+  gt_shash_element *shash_element = gt_shash_get_shash_element(shash,key);
   if (gt_expect_true(shash_element==NULL)) {
-    shash_element = malloc(sizeof(gt_shash_element));
-    gt_cond_fatal_error(!shash_element,MEM_ALLOC);
-    register const uint64_t key_length = strlen(key);
+    shash_element = gt_alloc(gt_shash_element);
+    const uint64_t key_length = strlen(key);
     shash_element->key = gt_strndup(key,key_length);
     shash_element->element = element;
     HASH_ADD_KEYPTR(hh,shash->shash_head,shash_element->key,key_length,shash_element);
@@ -97,11 +104,10 @@ GT_INLINE char* gt_shash_insert_object(
   GT_HASH_CHECK(shash);
   GT_NULL_CHECK(key); GT_NULL_CHECK(object);
   GT_NULL_CHECK(element_dup_fx); GT_NULL_CHECK(element_free_fx);
-  register gt_shash_element *shash_element = gt_shash_get_shash_element(shash,key);
+  gt_shash_element *shash_element = gt_shash_get_shash_element(shash,key);
   if (gt_expect_true(shash_element==NULL)) {
-    shash_element = malloc(sizeof(gt_shash_element));
-    gt_cond_fatal_error(!shash_element,MEM_ALLOC);
-    register const uint64_t key_length = strlen(key);
+    shash_element = gt_alloc(gt_shash_element);
+    const uint64_t key_length = strlen(key);
     shash_element->key = gt_strndup(key,key_length);
     shash_element->element = object;
     HASH_ADD_KEYPTR(hh,shash->shash_head,shash_element->key,key_length,shash_element);
@@ -155,7 +161,7 @@ GT_INLINE uint64_t gt_shash_get_num_elements(gt_shash* const shash) {
  * Miscellaneous
  */
 GT_INLINE gt_shash* gt_shash_dup(gt_shash* const shash) {
-  register gt_shash* const shash_cp =  gt_shash_new();
+  gt_shash* const shash_cp =  gt_shash_new();
   gt_shash_copy(shash_cp,shash);
   return shash_cp;
 }
@@ -165,7 +171,7 @@ GT_INLINE void gt_shash_copy(gt_shash* const shash_dst,gt_shash* const shash_src
     switch (shash_sh_element->element_type) {
       case GT_HASH_TYPE_REGULAR: {
         // Copy element
-        register void* const shash_element_cp = gt_malloc_(1,shash_sh_element->element_size,false,0);
+        void* const shash_element_cp = gt_malloc_(1,shash_sh_element->element_size,false,0);
         memcpy(shash_element_cp,shash_element,shash_sh_element->element_size);
         gt_shash_insert_primitive(shash_dst,skey,shash_element_cp,shash_sh_element->element_size);
         break;
@@ -180,3 +186,37 @@ GT_INLINE void gt_shash_copy(gt_shash* const shash_dst,gt_shash* const shash_src
     }
   } GT_SHASH_END_ITERATE;
 }
+
+/*
+ * Iterator
+ */
+GT_INLINE gt_shash_iterator* gt_shash_iterator_new(gt_shash* const shash) {
+  GT_HASH_CHECK(shash);
+  // Allocate
+  gt_shash_iterator* const shash_iterator = gt_alloc(gt_shash_iterator);
+  // Init
+  shash_iterator->shash = shash;
+  shash_iterator->next = shash;
+  return shash_iterator;
+}
+GT_INLINE void gt_shash_iterator_delete(gt_shash_iterator* const shash_iterator) {
+  GT_HASH_CHECK(shash_iterator->shash);
+  gt_free(shash_iterator);
+}
+GT_INLINE bool gt_shash_iterator_next(gt_shash_iterator* const shash_iterator) {
+  GT_HASH_CHECK(shash_iterator->shash);
+  if (gt_expect_false(shash_iterator->next==NULL)) return false;
+  shash_iterator->next = shash_iterator->next->hh.next;
+  return true;
+}
+GT_INLINE char* gt_shash_iterator_get_key(gt_shash_iterator* const shash_iterator) {
+  GT_HASH_CHECK(shash_iterator->shash);
+  GT_HASH_CHECK(shash_iterator->next);
+  return shash_iterator->next->key;
+}
+GT_INLINE void* gt_shash_iterator_get_element(gt_shash_iterator* const shash_iterator) {
+  GT_HASH_CHECK(shash_iterator->shash);
+  GT_HASH_CHECK(shash_iterator->next);
+  return shash_iterator->next->element;
+}
+
